function [opt,exit_opt1] = MAIN (yy , uu , nu, ww)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%              Eduardo Rodriguez Lopez                        %%%%%%
%%%%%%              Imperial College London                        %%%%%%
%%%%%%                Oct 2014                                     %%%%%%
%%%%%%                                                             %%%%%%
%%%%%%              Southampton University                         %%%%%%
%%%%%%                May 2017                                     %%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Eduardo Rodriguez-Lopez, Paul J.K. Bruce and Oliver R.H. Buxton: 'A  %%
% robust postprocessing method to extrapolate skin friction from the   %%
% velocity profile'. (2015) Experiments in Fluids 56(4):68             %%
%                                                                      %%
% Luis Blay Esteban, Eda Dogan, Eduardo Rodriguez-Lopez and Bharathram %%
% Ganapathisubramani: 'Skin friction measurements in a turbulent       %%
% boundary layer with zero-pressure-gradient under the influence of    %%
% free-stream turbulence. (2017) Experiments in Fluids XX(X):XX        %%
%                                                                      %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%
% This function fits an experimental velocity profile (provided in 'yy' & 
% 'uu' to an analytical description of the mean velocity profile. The inner
% and logarithmic regions are described by the Musker (1976) description
% while the wake can be described either by the exponential wake (smooth 
% boundary layers with laminar freestream see Chauhan et al., 2009) or by 
% Dean's wake in the case of turbulent boundary layers under freestream
% turbulent conditions. (See Hancock and Bradshaw 1983 - 1989)
%
% For a deeper description of the method and its validation agains
% experimental and numerical data, the user is refered to:
%
% Eduardo Rodriguez-Lopez, Paul J.K. Bruce and Oliver R.H. Buxton: 'A
% robust postprocessing method to extrapolate skin friction from the
% velocity profile'. (2015) Experiments in Fluids 56(4):68
%
% When using this program please cite the 2 aforementioned articles, THANKS
%
%
% INPUTS:
% yy  = A vector with the wall-normal locations in increasing order without
%       considering y=0. UNITS=milimiters [mm].
% uu  = A vector with the mean streamwise velocity corresponding to the
%       positions in yy. UNITS=meters per second [m/s]
% nu  = Kinematic viscosity of the fluid [m^2/s]
% ww  = The wake description, use 'exp' for Exponential wake and 'fst' for
%       wakes under freestream turbulence conditions.
%
% OUTPUTS:
% opt       = structure with 6 parameters. The 5 parameters of the fit 
%             (u_tau, kappa, delta_y, delta and PI) 
%             plus the value of s (equivalent to the constant B of the 
%             logarithmic law) described in the main reference 
% exit_opt1 = structure summarizing the outputs of the optimization
%             generated by Matlab
%
% Version 1.0 October 2014 Imperial College London. Eduardo Rodriguez Lopez
% Version 2.0 May 2017 Southampton University. Eduardo Rodriguez Lopez
%                      (inclusion of the FST wake)
%
% The program requires the optimization tool from MATLAB (fmincon)
%
%
%% Parameters
resolution_musk=3; %steps of 10^(-resolution_musker) in the wall-normal 
% integration of the musker velocity profile:
% the larger the resolution the more accurate values and slower program, 3
% is more than enough for almost every application and 2 would perform
% almost equally good but slightly quicker.


% b_s_k are the tabulated values of  kappa*B = 1.6[exp(0.1663B) ? 1].
% Equation 3 of the original reference of the method. This equation was
% first proposed by Nagib & Chauhan (2008) as referenced in the paper.
b_s_k=load('./b_s_k.mat','kappa','s_good');
% Note that the Musker velocity profile is developed as a function of 's'
% whereas this relationship is given in 'B'. There is a 1-to-1 equivalence
% between a given 's' in the Musker profile and the intercept of the
% log-law 'B' therefore the equation above can be written as a relationship
% between kappa and 's'. However, this relationsthip is not analytical
% hence it is tabulated after solving it numerically in 'b_s_k.mat'


%Removing points in y==0 in case they exist
if yy(1)==0
    yy=yy(2:end);
    uu=uu(2:end);
end

%getting an rough estimation of delta_99
for m=1:length(yy)
    if uu(m)>0.99*uu(end)
        delta_99=yy(m);
        break
    end
end

%% Bounds and intial values for optimization

%initial values
utau0=uu(end)/25; %Can be changed to a better estimation using Clauser plot
delta_y0=0;
kappa0=0.4;
delta0=1.3*delta_99;
PI0=0.5;

%lower bounds 
low_utau=0.5*utau0; 
low_deltay=-yy(1)+0.001;
low_kappa=0.2;
low_delta=0.8*delta_99; %In units of delta99
low_PI=-5;

%upper bounds
high_utau=1.5*utau0;
high_deltay=5; %(mm)
high_kappa=0.8;
high_delta=8*delta_99;
high_PI=5;

%ordering variables
alfa_ini=[utau0,delta_y0,delta0,PI0,kappa0];
alfa_low=[low_utau,low_deltay,low_delta,low_PI,low_kappa];
alfa_high=[high_utau,high_deltay,high_delta,high_PI,high_kappa];


%% selecting function to optimize

if strcmp(ww,'exp')
    f_to_opt=@(alfa)error_5_variable_bump_s...
        (alfa,yy,uu,b_s_k.s_good,b_s_k.kappa,resolution_musk,nu);
elseif strcmp(ww,'fst')
    f_to_opt=@(alfa)error_51_variable_bump_s...
        (alfa,yy,uu,b_s_k.s_good,b_s_k.kappa,resolution_musk,nu);
else 
    error('Wrong value of ww (type of wake should be "exp" or "fst")')
end



%% Call the optimization routine
[options] = set_optim_params ();
[opt_point,fval,flag,output,lambda,grad,hessian] = ...
    fmincon(f_to_opt,alfa_ini,[],[],[],[],alfa_low,alfa_high,[],options);

%% Rename and put together the output variables
exit_opt1.fval=fval;
exit_opt1.flag=flag;
exit_opt1.output=output;
exit_opt1.lambda=lambda;
exit_opt1.grad=grad;
exit_opt1.hessian=hessian;

opt.utau=opt_point(1);
opt.delta_y=opt_point(2);
opt.delta=opt_point(3);
opt.PI=opt_point(4);
opt.kappa=opt_point(5);
s5=interp1(b_s_k.kappa,b_s_k.s_good,opt_point(5),'linear');
opt.s=s5;



